@@ -12,8 +12,6 @@ import fire
 import platform
 import sys
 
-print(f"Executable: {sys.executable}; V: {sys.version}; {platform.python_build()}; CONDA_PREFIX: {os.getenv('CONDA_PREFIX')}")
-
 from nexinfosys.command_generators import Issue, IType
 from nexinfosys.common.helper import download_file, generate_json
 from nexinfosys.embedded_nis import NIS
@@ -60,6 +58,12 @@ def get_file_url(fn):
     return "file:" + urllib.request.pathname2url(fn)
 
 
+def print_sys_info(show_sys_info):
+    if show_sys_info:
+        print(f"Executable: {sys.executable}; V: {sys.version}; {platform.python_build()}; "
+              f"CONDA_PREFIX: {os.getenv('CONDA_PREFIX')}")
+
+
 def read_submit_solve_nis_file(nis_file_url: str, state: State, solve=False) -> Tuple[NIS, List[Issue]]:
     nis = NIS()
     nis.open_session(True, state)
@@ -199,7 +203,7 @@ def print_issues(operation, file, issues: List[Issue], msg_in_case_of_error: str
     else:
         print(
             f"File '{file}' was already cached, meaning no change and no errors detected. "
-            f"Please, add --force_refresh=True to force reprocessing.")
+            f"Please, add --force-refresh to force reprocessing.")
 
 
 def write_results(state_or_nis: Union[State, NIS], output_dir: str, datasets: List[str]):
@@ -210,32 +214,36 @@ def write_results(state_or_nis: Union[State, NIS], output_dir: str, datasets: Li
         nis = state_or_nis
 
     # Convert to required format
-    datasets = datasets.split(",") if datasets.strip() != "" else []
-    _ = []
-    for ds in datasets:
-        tmp = ds.strip().split(".")
-        if len(tmp) == 3:
-            _.append([tmp[0], tmp[1]+"."+tmp[2]])
-        elif len(tmp) == 2:
-            _.append([None, tmp[0] + "." + tmp[1]])
-        else:
-            print(f"Could not parse dataset request entry: {ds}")
-    tmp = nis.get_results(_)
-    any_error = False
-    for i, ds in enumerate(tmp):
-        name = datasets[i]
-        if ds[2]:
-            if isinstance(ds[0], StringIO):
-                val = ds[0].getvalue()
+    if datasets:
+        datasets = datasets.split(",") if datasets.strip() != "" else []
+        _ = []
+        for ds in datasets:
+            tmp = ds.strip().split(".")
+            if len(tmp) == 3:
+                _.append([tmp[0], tmp[1]+"."+tmp[2]])
+            elif len(tmp) == 2:
+                _.append([None, tmp[0] + "." + tmp[1]])
             else:
-                val = ds[0]
-            flag = "b" if not isinstance(ds[0], StringIO) and not isinstance(ds[0], str) else "t"
-            print(f"Writing {name} to: {output_dir}{os.sep}{name}")
-            with open(f"{output_dir}{os.sep}{name}", f"w{flag}") as f:
-                f.write(val)
-        else:
-            any_error = True
-            print(f"Could not retrieve dataset: {name}")
+                print(f"Could not parse dataset request entry: {ds}")
+        tmp = nis.get_results(_)
+        any_error = False
+        for i, ds in enumerate(tmp):
+            name = datasets[i].strip()
+            if ds[2]:
+                if isinstance(ds[0], StringIO):
+                    val = ds[0].getvalue()
+                else:
+                    val = ds[0]
+                flag = "b" if not isinstance(ds[0], StringIO) and not isinstance(ds[0], str) else "t"
+                print(f"Writing {name} to: {output_dir}{os.sep}{name}")
+                with open(f"{output_dir}{os.sep}{name}", f"w{flag}") as f:
+                    f.write(val)
+            else:
+                any_error = True
+                print(f"Could not retrieve dataset: {name}")
+    else:
+        any_error = True
+        tmp = []
     if any_error or len(tmp) == 0:
         print("Reference of available datasets: --------")
         lst = nis.query_available_datasets()
@@ -286,7 +294,7 @@ nexinfosys solve https://docs.google.com/spreadsheets/d/1C5xNGvdORWqrWL6Ux2nBXMd
 nexinfosys solve https://docs.google.com/spreadsheets/d/1C5xNGvdORWqrWL6Ux2nBXMdX51ktEDJCmFdDrbFrhX4/edit#gid=0 /home/rnebot/tmp/example1withValues --datasets="flow_graph_solution.csv"
 
     """
-    def parse(self, file: str, work_path: str, datasets: str = "", force_refresh: bool = False, log: str = None):
+    def parse(self, file: str, work_path: str, datasets: str = "", force_refresh: bool = False, log: str = None, sys_info: bool = False):
         """
         Parse and retrieve datasets
 
@@ -295,15 +303,17 @@ nexinfosys solve https://docs.google.com/spreadsheets/d/1C5xNGvdORWqrWL6Ux2nBXMd
         :param datasets: Comma separated list of datasets to export
         :param force_refresh: True to force parsing even if the main file has not changed
         :param log: Set log level to one of: Error (E, Err), Debug (D), Warning (W, Warn), Info (I), Off, Critical (Fatal)
+        :param sys_info: Print system information
         :return:
         """
+        print_sys_info(sys_info)
         set_log_level_from_cli_param(log)
         os.makedirs(work_path, exist_ok=True)
         state, _, issues = prepare_base_state(file, False, work_path, force_refresh)
         print_issues("Parsing", file, issues)
         write_results(state, work_path, datasets)
 
-    def solve(self, file: str, work_path: str, datasets: str = "", force_refresh: bool = False, log: str = None):
+    def solve(self, file: str, work_path: str, datasets: str = "", force_refresh: bool = False, log: str = None, sys_info: bool = False):
         """
         Parse, solve and retrieve datasets
 
@@ -312,8 +322,10 @@ nexinfosys solve https://docs.google.com/spreadsheets/d/1C5xNGvdORWqrWL6Ux2nBXMd
         :param datasets: Comma separated list of datasets to export
         :param force_refresh: True to force parsing and solving even if the main file has not changed
         :param log: Set log level to one of: Error (E, Err), Debug (D), Warning (W, Warn), Info (I), Off, Critical (Fatal)
+        :param sys_info: Print system information
         :return:
         """
+        print_sys_info(sys_info)
         set_log_level_from_cli_param(log)
         os.makedirs(work_path, exist_ok=True)
         state, _, issues = prepare_base_state(file, True, work_path, force_refresh)
@@ -321,7 +333,8 @@ nexinfosys solve https://docs.google.com/spreadsheets/d/1C5xNGvdORWqrWL6Ux2nBXMd
         write_results(state, work_path, datasets)
 
     def solve_dynamic_scenario(self, file: str, work_path: str, params_file_path: str = "",
-                               datasets: str = "", force_refresh: bool = False, log: str = None):
+                               datasets: str = "", force_refresh: bool = False,
+                               log: str = None, sys_info: bool = False):
         """
         Run (solve) dynamic scenario and get output datasets
 
@@ -331,8 +344,10 @@ nexinfosys solve https://docs.google.com/spreadsheets/d/1C5xNGvdORWqrWL6Ux2nBXMd
         :param datasets: Comma separated list of datasets to export
         :param force_refresh: True to force parsing and solving even if the main file has not changed
         :param log: Set log level to one of: Error (E, Err), Debug (D), Warning (W, Warn), Info (I), Off, Critical (Fatal)
+        :param sys_info: Print system information
         :return:
         """
+        print_sys_info(sys_info)
         set_log_level_from_cli_param(log)
         os.makedirs(work_path, exist_ok=True)
         state, _, issues = prepare_base_state(file, True, work_path, force_refresh)
