@@ -6,6 +6,7 @@ https://gist.github.com/cynici/5865326
 
 """
 import importlib
+import itertools
 import math
 import re
 from typing import Dict, Tuple, Union, List
@@ -283,78 +284,6 @@ def obtain_subset_of_processors(processors_selector: str, serialized_model: lxml
     return results, processors
 
 
-def aggregator_generic(funct, field: str, xquery: str = None, scope: str = 'Total', processors_dom=None,
-                       processors_map=None, df_group=None, df_indicators_group=None):
-    """
-    AGGREGATE "field" for all processors meeting the XQuery and scope, applying aggregator "funct"
-    :param field:
-    :param xquery:
-    :param processors_dom:
-    :param processors_map:
-    :param df_group:
-    :return:
-    """
-    inv_map = {v: k for k, v in processors_map.items()}
-    processors = obtain_processors(xquery, processors_dom, processors_map)
-    processor_names = set([inv_map[p] for p in processors])
-    # Only one Interface(and its Orientation) allowed
-    # Filter the passed group by processor and scope, by Interface and Orientation
-    # Aggregate the Value column according of remaining rows
-    f = field.lower()
-    orient = ["Input", "Output"]
-    if field.lower().endswith("_input"):
-        f = f[:-len("_input")]
-        orient = ["Input"]
-    elif field.lower().endswith("_output"):
-        f = f[:-len("_output")]
-        orient = ["Output"]
-    filt = df_group.index.get_level_values("Processor").isin(processor_names) & \
-           df_group.index.get_level_values("Scope").isin([scope]) & \
-           df_group.index.get_level_values("Interface").isin([f]) & \
-           df_group.index.get_level_values("Orientation").isin(orient)
-    values = df_group[filt]["Value"].values
-    if len(values) > 0:
-        return funct(values)
-    else:
-        return None
-
-
-def aggregator_sum(field: str, xquery: str = None, scope: str = 'Total', processors_dom=None, processors_map=None,
-                   df_group=None, df_indicators_group=None):
-    return aggregator_generic(np.nansum, field, xquery, scope, processors_dom, processors_map, df_group,
-                              df_indicators_group)
-
-
-def aggregator_avg(field: str, xquery: str = None, scope: str = 'Total', processors_dom=None, processors_map=None,
-                   df_group=None, df_indicators_group=None):
-    return aggregator_generic(np.nanavg, field, xquery, scope, processors_dom, processors_map, df_group,
-                              df_indicators_group)
-
-
-def aggregator_max(field: str, xquery: str = None, scope: str = 'Total', processors_dom=None, processors_map=None,
-                   df_group=None, df_indicators_group=None):
-    return aggregator_generic(np.nanmax, field, xquery, scope, processors_dom, processors_map, df_group,
-                              df_indicators_group)
-
-
-def aggregator_min(field: str, xquery: str = None, scope: str = 'Total', processors_dom=None, processors_map=None,
-                   df_group=None, df_indicators_group=None):
-    return aggregator_generic(np.nanmin, field, xquery, scope, processors_dom, processors_map, df_group,
-                              df_indicators_group)
-
-
-def aggregator_count(field: str, xquery: str = None, scope: str = 'Total', processors_dom=None, processors_map=None,
-                     df_group=None, df_indicators_group=None):
-    return aggregator_generic(lambda v: len(v), field, xquery, scope, processors_dom, processors_map, df_group,
-                              df_indicators_group)
-
-
-def aggregator_nan_count(field: str, xquery: str = None, scope: str = 'Total', processors_dom=None, processors_map=None,
-                         df_group=None, df_indicators_group=None):
-    return aggregator_generic(lambda v: sum(np.isnan(v)), field, xquery, scope, processors_dom, processors_map,
-                              df_group, df_indicators_group)
-
-
 # Comparison operators
 opMap = {
     "<": lambda a, b: a < b,
@@ -382,7 +311,7 @@ def ast_evaluator(exp: Dict, state: State, obj, issue_lst, atomic_h_names=False,
     :param issue_lst: List in which issues have to be annotated
     :param atomic_h_names: If True, treat variable names as atomic (False processes them part by part, from left to right). Used in dataset expansion
     :param allowed_functions: Set of allowed function names (and their definitions)
-    :param account_nas_name: If True, each evaluation at top level returns 4 results: the evaluation, number of variables,
+    :param account_nas_name: If != "", each evaluation at top level returns 4 results: the evaluation, number of variables,
                         number of not available and number of not applicable (i.e., like zero)
     :param nest_level: Recursive level of the function. Used to allow evaluation of addends when they are NotAvailable
     :return: value (scalar EXCEPT for named parameters, which return a tuple "parameter name - parameter value"), list of unresolved variables
@@ -442,22 +371,37 @@ def ast_evaluator(exp: Dict, state: State, obj, issue_lst, atomic_h_names=False,
             else:
                 ret_val = None
         elif t == "function":  # Call function
-            # First, obtain the Parameters
-            args = []
-            kwargs = {}
-            can_resolve = True
-            for p in [ast_evaluator(p, state, obj, issue_lst, atomic_h_names, allowed_functions,
-                                    account_nas_name=account_nas_name, nest_level=nest_level + 1) for p in
-                      exp["params"]]:
-                if len(p) == 3:
-                    kwargs[p[0]] = p[1]
-                    tmp = p[2]
+            # If it is a function (not a method), is it a valid function, is it an aggregator function?
+            if obj is None:
+                if exp["name"] in allowed_functions:
+                    _f = allowed_functions[exp["name"]]
+                    aggregate = _f.get("aggregate", False)
                 else:
-                    args.append(p[0])
-                    tmp = p[1]
-                involved_vars.update(tmp)
-                if len(tmp) > 0:
-                    can_resolve = False
+                    aggregate = False
+                    issue_lst.append((3, "Function '" + exp["name"] + "' does not exist"))
+
+            # First, obtain the Parameters
+            args = []  # List of unnamed parameters
+            kwargs = {}  # List of named parameters
+            can_resolve = True
+            for i, p in enumerate(exp["params"]):
+                # If it is an aggregator function, first parameter not evaluated, evaluated by the aggregator function
+                if aggregate and i == 0:
+                    # First parameter is not evaluated, the AST is just passed to the aggregator function,
+                    # which will evaluate it for every matching processor (per scenario, period)
+                    args.append(p)
+                else:
+                    q = ast_evaluator(p, state, obj, issue_lst, atomic_h_names, allowed_functions,
+                                      account_nas_name=account_nas_name, nest_level=nest_level + 1)
+                    if len(q) == 3:  # Named parameter
+                        kwargs[q[0]] = q[1]
+                        tmp = q[2]
+                    else:  # Parameter
+                        args.append(q[0])
+                        tmp = q[1]
+                    involved_vars.update(tmp)
+                    if len(tmp) > 0:
+                        can_resolve = False
 
             if obj is None:
                 # Check if it can be resolved (all variables specified)
@@ -477,14 +421,16 @@ def ast_evaluator(exp: Dict, state: State, obj, issue_lst, atomic_h_names=False,
                             kwargs[name] = state.get("_processors_map")
                         elif sp_kwarg == "DataFrameGroup":
                             kwargs[name] = state.get("_df_group")
-                        elif sp_kwarg == "IndicatorsDataFrameGroup":
-                            kwargs[name] = state.get("_df_indicators_group")
+                        elif sp_kwarg == "IndicatorDictionaries":
+                            kwargs[name] = state.get("_indicators_tmp")
                         elif sp_kwarg == "IndicatorState":
                             kwargs[name] = state
                         elif sp_kwarg == "LCIAMethods":
                             kwargs[name] = state.get("_lcia_methods")
                         elif sp_kwarg == "AccountNA":
                             kwargs[name] = account_nas_name
+                        elif sp_kwarg == "ProcessorNames":
+                            kwargs[name] = state.get("_processor_names")
 
                     # CALL FUNCTION!!
                     try:
@@ -522,7 +468,7 @@ def ast_evaluator(exp: Dict, state: State, obj, issue_lst, atomic_h_names=False,
                             else:
                                 involved_vars[o] = "NAp"
                             if account_nas_name:
-                                obj = 0  # If not defined, the value is zero
+                                obj = 0  # If not defined, the value is zero (NAs is for Sums; for Products it would probably be 1...)
                         else:  # Value defined, check if it is a NAv (NaN)
                             if isinstance(obj, (float, int)):  # AccountNAs is valid only for float's
                                 if math.isnan(obj):
